#!/usr/bin/python
import os
import stat
import sys

linux = "linux_amd64"

if (len(sys.argv) == 2 and sys.argv[1] not in ["linux_amd64", "linux_i386"]) or len(sys.argv) > 2:
    print "usage: make_apps [linux_amd64|linux_i386]"
    sys.exit(1)

if len(sys.argv) == 2:
    linux = sys.argv[1]

includes = {
    "weka": {"jars": ["$SRLDB_HOME/../WEKA/bin", "$SRLDB_HOME/lib/weka_fipm.jar"]},
    "srldb": {"jars": ["$SRLDB_HOME/bin", "$SRLDB_HOME/lib/srldb.jar", "$SRLDB_HOME/../TUMUtils/bin", "$SRLDB_HOME/lib/tumutils.jar"]},
    "jython": {"jars": ["$SRLDB_HOME/lib/jython22.jar"]},
    "swt": {"jars": ["$SRLDB_HOME/lib/swt_<ARCH>/swt.jar"], "lib":"$SRLDB_HOME/lib/swt_<ARCH>"},
    "bnj": {"jars": ["$SRLDB_HOME/../BNJ/bin", "$SRLDB_HOME/lib/bnj.jar", "$SRLDB_HOME/lib/bnj_res.jar"]},
    "smile": {"jars": ["$SRLDB_HOME/lib/jsmile_<ARCH>/smile.jar"], "lib": "$SRLDB_HOME/lib/jsmile_<ARCH>"},
    "log4j": {"jars": ["$SRLDB_HOME/lib/log4j-1.2.9.jar"]},
    "proximity": {"jars": ["$SRLDB_HOME/lib/proximity.jar"]},
    "ssj": {"jars": ["$SRLDB_HOME/lib/ssj.jar", "$SRLDB_HOME/lib/optimization.jar"]},
    "jahmm": {"jars": ["$SRLDB_HOME/../jahmm/bin", "$SRLDB_HOME/lib/jahmm.jar"]},
    "processing": {"jars": ["$SRLDB_HOME/lib/processing.org/core.jar"]},
    "mysql": {"jars": ["$SRLDB_HOME/lib/mysql-connector-java-3.0.11-stable-bin.jar"]},
    "vecmath": {"jars": ["$SRLDB_HOME/lib/vecmath.jar"]},
    "yprolog": {"jars": ["$SRLDB_HOME/lib/yprolog.jar"]},
    "kipm": {"jars": ["$SRLDB_HOME/../KIPM/bin", "$SRLDB_HOME/lib/kipm.jar", "$SRLDB_HOME/lib/fipmbase.jar", "$SRLDB_HOME/lib/kipmdata.jar"]},
}

java_apps = {
    "BLNinfer": {"class": "edu.tum.cs.srl.bayesnets.inference.BLNinfer", "includes": ["srldb", "weka", "jython", "swt", "bnj", "log4j", "smile", "ssj", "yprolog"]},
    "BLN2MLN": {"class": "BLN2MLN", "includes": ["srldb", "weka", "jython", "swt", "bnj", "log4j"]},
    "BNinfer": {"class": "BNinfer", "includes": ["srldb", "swt", "bnj", "log4j", "smile"]},
    "BLOG2MLN": {"class": "BLOG2MLN", "includes": ["srldb", "weka", "swt", "bnj", "log4j"]},
    "BN2CSV": {"class": "BN2CSV", "includes": ["srldb", "swt", "bnj", "log4j"]},
    "BNListCPTs": {"class": "BNListCPTs", "includes": ["srldb", "swt", "bnj", "log4j"]},
    "BNrandomEvidence": {"class": "BNrandomEvidence", "includes": ["srldb", "swt", "bnj", "log4j"]},
    "MLN2WCSP": {"class": "MLN2WCSP", "includes": ["srldb", "log4j", "jython"]},
    "MLNinfer": {"class": "MLNinfer", "includes": ["srldb", "bnj", "log4j", "jython"]},
    "bnj": {"class": "BNJ", "includes": ["srldb", "bnj", "swt", "log4j"]},
    "genDB": {"class": "genDB", "includes": ["srldb", "jython"]},
    "groundBLOG": {"class": "groundBLOG", "includes": ["srldb", "bnj"]},
    "learnABL": {"class": "learnABL", "includes": ["srldb", "weka", "swt", "bnj", "log4j", "yprolog"]},
    "learnBLOG": {"class": "learnBLOG", "includes": ["srldb", "weka", "swt", "bnj", "log4j", "yprolog"]},
    "fixSRLDB": {"class": "fixSRLDB", "includes": ["srldb", "proximity"]},
    "jython": {"class": "-jar $SRLDB_HOME/lib/jython251.jar", "includes": []},
    "TrajectoryVis": {"class": "edu.tum.cs.analysis.actionrecognition.mocap.TrajVis", "includes": ["srldb", "kipm", "processing", "jahmm", "mysql", "vecmath"]},
    "syprolog": {"class": "PrologShell", "includes": ["srldb", "yprolog"]},
    "yprolog": {"class": "yprolog.Go", "includes": ["yprolog"]},
}

python_apps = [
    {"name": "mlnquery", "script": "$SRLDB_HOME/python/queryTool.py"},
    {"name": "mlnlearn", "script": "$SRLDB_HOME/python/learningTool.py"},
    {"name": "blnquery", "script": "$SRLDB_HOME/python/blnQueryTool.py"},
    {"name": "bnquery", "script": "$SRLDB_HOME/python/bnQueryTool.py"},
    {"name": "blnlearn", "script": "$SRLDB_HOME/python/blnLearningTool.py"},
    {"name": "fixCR", "script": "$SRLDB_HOME/python/fixCR.py"},
    {"name": "MLN", "script": "$SRLDB_HOME/python/MLN.py"},
    {"name": "trajvis", "script": "$SRLDB_HOME/python/trajvis.py"},
    {"name": "evalSeqLabels", "script": "$SRLDB_HOME/python/evalSeqLabels.py"},
]

def adapt(name, arch):
    return name.replace("/", os.path.sep).replace("<ARCH>", arch).replace("$SRLDB_HOME", os.path.abspath("."))    

def getJavaAppData(name, arch):
    jars = []
    libs = []
    skip = False
    for inc in java_apps[name]["includes"]:
        libjars = map(lambda jar: adapt(jar, arch), includes[inc]["jars"])
        if len(filter(os.path.exists, libjars)) == 0:
            skip = True                    
        jars.extend(libjars)
        if "lib" in includes[inc]:
            libs.append(adapt(includes[inc]["lib"], arch))
    return {"cp": jars, "lib": libs, "skip": skip}

if __name__ == '__main__':
    
    if not os.path.exists("apps"):
        os.mkdir("apps")
    
    for arch in [linux, "win32"]:
        print "\nCreating application files for %s..." % arch
        isWindows = "win" in arch
        allargs = "%c*" % {True:"%", False:"$"}[isWindows]
        for appname, app in java_apps.iteritems():
            filename = os.path.join("apps", "%s%s" % (appname, {True:".bat", False:""}[isWindows]))
            data = getJavaAppData(appname, arch)
            print "  %s" % filename
            if data["skip"]:
                print "    skipped because some dependencies are not included in this distribution"
                continue            
            if len(data["lib"]) > 0:
                libpath = '"-Djava.library.path=%s"' % os.path.pathsep.join(data["lib"])
            f = file(filename, "w")
            if not isWindows: f.write("#!/bin/sh\n")
            f.write('java -cp "%s" %s %s %s\n' % (os.path.pathsep.join(data["cp"]), libpath, adapt(app["class"], arch), allargs))
            f.close()
            if not isWindows: os.chmod(filename, stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR | stat.S_IRGRP | stat.S_IXGRP | stat.S_IROTH | stat.S_IXOTH)
        for app in python_apps:
            filename = os.path.join("apps", "%s%s" % (app["name"], {True:".bat", False:""}[isWindows]))
            print "  %s" % filename
            f = file(filename, "w")
            if not isWindows: f.write("#!/bin/sh\n")
            f.write("python -O %s %s\n" % (adapt(app["script"], arch), allargs))
            f.close()
            if not isWindows: os.chmod(filename, stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR | stat.S_IRGRP | stat.S_IXGRP | stat.S_IROTH | stat.S_IXOTH)
    
    if len(sys.argv) == 1:
        print "\nNote: To create application files for Linux/i386, pass \"linux_i386\" as a parameter!\n"