'''
Edited version for October 2009 workshop - added tea box and napkins
'''

'''
Generator for meals and persons participating in them

One meal per day is generated for a given number of weeks. 6 different people participate in those meals depending on the day, the meal is held. (e.g.: Anna is taking part in any meal on Monday, Tuesday and Wednesday, Bert on Monday, Wednesday and with a probability of 80% each on Tuesday and Saturday)

For every person there exists a distribution over the time the person wants the meal to take place. The actual time of the meal is chosen by the combined distribution of all participating persons.

There exist a given number of places at the breakfast table and every person has a favorite place to sit at. However, if two person with the same favorite place take part in the same meal, only one person (randomly selected) can be seated in his favorite place while the other person takes any other free one.

For every type of utensils there exists a given number ob object with this type. Whenever a object of a given type is needed, it is chosen at random from its container. For a specific good, however, there exists only one constant object with the attribute of being a specific good (e.g. coffee).

Every person has different distribution over the utensils it uses and the goods it consumes in a meal, e.g.:
	Anna always uses a cup and with a probability of 90% it's for coffee. She uses a plate in 95% and a bowl otherwise. Also, she doesn't eat meat.
	Bert always uses a cup and with probability of 90% it's for tea.
	Charly uses a bowl with probability of 95%, also he only drinks water when using a glass
	Dorothy uses a glass with probability of 80%, a cup only with probability of 20%\
	Emily uses both a plate and a bowl with probability of 80% each and a cup and a glass with 90% each. She drinks coffee and juice with probability of 90%
	Frank is like Emily escept that he is drinking tea and water with probability of 90%

Two community goods (a platter and a pitcher) are used by all participants with probability of 80% each.
	
	
Generator extended with Dinner and Lunch (though Time Distributions need to be diversified
Generator now uses gen_template
'''

import sys
sys.path.append("/usr/wiss/jain/work/code/SRLDB/datagen")
#sys.path.insert(0, "D:\SRLDB\SRLDB\python")
sys.path.append("..")
from gen_classes import *
from datagen import *

numUtensils = 6
numSeats = 4
numWeeks = 10
peopleAsConstants = False # False = people as objects, i.e. "Anna" is the "name" attribute of an object rather than the constant referring to the object itself

def strBool(x):
	return {0: "false", 1: "true"}[int(x)]

class PersonNEW(Person):
		def __init__(self,world,type,hasFixedConstantName):
			Person.__init__(self,world,type,hasFixedConstantName)
	
		def setFavoriteSeat(self,favSeat):
			self.favSeat = favSeat
	
		def setConsumptionLinks(self, meal, usageT = 'standard', consumptionT = 'standard', forin = False):
			#sets links between persons, utensils/goods and meals. There are different kinds of predicates in use, the predicate type depends on the arguments of usageT and consumptionT:
			# consumptionT / usageT:
				# 'standard': The world consists of many objects of the same type, one is chosen from the container and thereby deleted, so no one else can use the object
				# 'CAO' (Constant as Object): The world consists only of one object for each objectT (which uses the type name of the object directly as the constant name)
			# the forin - attribute decides, whether the predicate 'usesForIn' is generated - a predicate that links utensils to goods.
			mealT = meal['mealT']
			if forin:
				self.uses, self.consumes, self.usesfor = self.sampleMeal(mealT,True)
			else:
				self.uses, self.consumes = self.sampleMeal(mealT,False)
			utensilDict = {}
			goodDict = {}
			# consumption of goods
			if consumptionT == 'standard':
				for good in self.consumes:
					consGood = Chooser(self.world,good).choose()
					self.linkto("consumesIn", consGood, meal)
					self.linkto("consumesAnyIn", consGood["goodT"], meal)
					goodDict[good] = consGood
			elif consumptionT == 'CAO':
				for good in self.consumes:
					consGood = self.world.getContainer(good).items()[0]
					self.linkto("consumesAnyIn", consGood, meal)
					goodDict[good] = consGood
			# utensil usage
			if usageT == 'standard':
				for utensilT in self.uses:
					utensil =  Chooser(self.world,utensilT).choose()
					self.linkto("usesIn", utensil, meal)
					self.linkto("usesAnyIn", utensil["utensilT"], meal)
					utensilDict[utensilT] = utensil
			elif usageT == 'CAO':
				for utensil in self.uses:
					usedUt = self.world.getContainer(utensil).items()[0]
					self.linkto("usesAnyIn", usesUt, meal)
					utensilDict[utensil] = usesUt
			# link objects to the goods they are used for
			if forin:
				for utensil in self.usesfor:
					for good in self.usesfor[utensil]:
						self.linkto('usesForIn', utensilDict[utensil], goodDict[good], meal)
						self.linkto('usesAnyForIn', utensilDict[utensil]["utensilT"], goodDict[good], meal)
		
		def takesPartIn(self,meal,persons,seats):
			''' makes a person take part in a meal, selecting the seat '''
			weekend = meal["day"] >= 5
			self.time = self.returnPreferredTime(meal['mealT'],weekend)
			seat = self.favSeat
			seatT = self.favSeat.toInt()
			people = reduce(lambda x,y: x+y, persons.values())
			i = seatT
			while True:
				seat = seats['Seat%d' % i][0]
				placeRob = filter(lambda x: x.hasLink(Link("sitsAtIn",x,seat,meal)), people) # get the people who sit in the wanted seat
				if placeRob == []:
					break
				i = (i+1) % numSeats
			self.linkto("sitsAtIn",seat,meal)
			self.linkto("takesPartIn",meal)
		def getUsedUtensils(self):
			if self.uses != None:
				return self.uses		
		def getConsumedGoods(self):
			if self.consumes != None:
				return self.consumes
 
class MealGen(ObjectGenerator):
	def __init__(self, world,persons,seats):
		ObjectGenerator.__init__(self, "meal", world.getContainer("event"), {})
		self.world = world
		self.persons = persons
		self.seats = seats

	def _createAttributes(self, obj, **args):
		days = ["Monday","Tuesday","Wednesday","Thursday","Friday","Saturday","Sunday"]
		weekend = args["day"] >= 5
		obj.setAttr("day", days[args["day"]])
		obj.setAttr("weekend", strBool(weekend))
		obj.setAttr("mealT", args["type"])
			
	def _createLinks(self, event, **args):
		world = self.world
		if event['mealT'] == 'Breakfast':
			times = {"earlyMorning": 0.0, "lateMorning": 0.0, "noon": 0.0}
		elif event['mealT'] == 'Lunch':
			times = {"noon": 0.0, "earlyAfternoon": 0.0}
		elif event['mealT'] == 'Dinner':
			times = {"lateAfternoon": 0.0, "evening": 0.0}
		
		# determine people who participate depending on day of week
		if event.getAttr("day") == "Monday":
			participate = ["Anna","Bert"]
			if decide(0.4): participate.append("Charly")
		elif event.getAttr("day") == "Tuesday":
			participate = ["Dorothy","Frank","Anna"]
			if decide(0.8): participate.append("Bert")
		elif event.getAttr("day") == "Wednesday":
			participate = ["Anna","Bert"]
			if decide(0.8): participate.append("Dorothy")
			if decide(0.8): participate.append("Emily")
		elif event.getAttr("day") == "Thursday":
			participate = ["Dorothy","Emily","Charly"]
		elif event.getAttr("day") == "Friday":
			participate = ["Charly","Emily"]
			if decide(0.8): participate.append("Dorothy")
		elif event.getAttr("day") == "Saturday":
			participate = ["Emily","Frank"]
			if decide(0.8): participate.append("Charly")
			if decide(0.8): participate.append("Bert")
		elif event.getAttr("day") == "Sunday":
			participate = ["Frank","Dorothy"]
			if decide(0.8): participate.append("Emily")
			
		# determine time of meal by building a distribution using the preferred times of the participants
		#check, whether a tea box is used
		teaboxUsage = False
		for person in participate:
			self.persons[person][0].takesPartIn(event,self.persons,self.seats)
			self.persons[person][0].setConsumptionLinks(event,'standard','CAO',True)
			#handle teabox usage
			if 'tea' in self.persons[person][0].getConsumedGoods():
				teaboxUsage = True
			#handle napkin usage
			napkinProb = 0.5
			if 'pizza' in self.persons[person][0].getConsumedGoods():
				napkinProb = 0.7
			if decide(napkinProb):
				utensil =  Chooser(self.world,'napkin').choose()
				self.persons[person][0].linkto("usesIn", utensil,event)
				self.persons[person][0].linkto("usesAnyIn", utensil["utensilT"], event)
			times[self.persons[person][0].time] += 1
		event["timeT"] = AttrDist(times).generate()
		
		# determine usage of shared objects
		pitcher = Chooser(world,'pitcher').choose()
		platter = Chooser(world,'platter').choose()
		teabox = Chooser(world,'teabox').choose()  
		if decide(0.8): 
			pitcher.linkto("usedByAllIn",event)
			ConstantAsObject(pitcher["utensilT"], self.world).linkto("anyUsedByAllIn", event)
		if decide(0.8): 
			platter.linkto("usedByAllIn",event)
			ConstantAsObject(platter["utensilT"], self.world).linkto("anyUsedByAllIn", event)
		if teabox:
			teabox.linkto("usedByAllIn",event)
			ConstantAsObject(teabox["utensilT"], self.world).linkto("anyUsedByAllIn", event)
		
		world.restoreAllRemovedObjects()


# --- main app ---

world = World()

# generate places
seats = {}
for i in range(numSeats):
	seatT = "Seat" +str(i)
	world.addContainer(seatT)
	seats[seatT] = []    
	seats[seatT].append(Seat(world,seatT,True))

# generate kitchen utensils
utensilTypes = ["cup", "glass", "bowl", "plate", "spoon", "fork", "knife","napkin"]
utensils = {}
for utensilT in utensilTypes:
	world.addContainer(utensilT)
	utensils[utensilT] = []
	for i in range(numUtensils):            
		utensils[utensilT].append(Utensil(world,utensilT))

#community utensils
comUtensilTypes = ["pitcher","platter","teabox"]
for comUtensilT in comUtensilTypes:
	world.addContainer(comUtensilT)
	utensils[comUtensilT] = []
	utensils[comUtensilT].append(Utensil(world,comUtensilT))
		   
# generate goods
goodsTypes = ["cheese","sausage", "cereals", "bread", "coffee", "tea", "water", "juice", 'pizza', 'noodles', 'salad', 'soup', 'cake']
goods = {}
for goodsT in goodsTypes:
	world.addContainer(goodsT)
	goods[goodsT] = []
	goods[goodsT].append(Good(world,goodsT,True))

# generate kitchen users
peopleT = ["Anna","Bert","Charly","Dorothy","Emily","Frank"]
people = {}
i = 1
for personName in peopleT:
	people[personName] = []
	world.addContainer(personName)
	person = eval('%s(world, PersonNEW, peopleAsConstants)' % personName)
	person.setFavoriteSeat(seats['Seat'+str(i%numSeats)][0])
	person["name"] = personName
	people[personName].append(person)    
	i += 1

# generate events    
events = world.addContainer("event")
mealgen = MealGen(world,people,seats)
mealTypes = ['Breakfast','Lunch','Dinner']
for day in range(7*numWeeks):
	for mT in mealTypes:
		mealgen.generate(day=day % 7, type=mT)

# print all generated objects
world.printAll()

# return the database
db = world.getDatabase()
